<?php

/**
 * Интеграция платежного шлюза RBS с CMS интернет-магазинов
 * @author RBS
 * @version 2.0
 */
class RBS
{
    /**
     * АДРЕС ТЕСТОВОГО ШЛЮЗА
     * @var string
     */
    const API_TEST_URL = "https://3dsec.sberbank.ru/payment/rest/";

    /**
     * АДРЕС БОЕВОГО ШЛЮЗА
     * @var string
     */
    const API_PROD_URL = "https://securepayments.sberbank.ru/payment/rest/";
    /**
     * Название для лог-файла
     * @var string
     */
    const log_file = 'rbs_payment.log';
    /**
     * Название CMS
     * @var string
     */
    public $cms_name;
    /**
     * Версия CMS
     * @var string
     */
    public $cms_version;
    /**
     * Версия модуля
     * @var string
     */
    public $module_version = 'd2.0';
    /**
     * Отправка корзины покупателя на шлюз
     * Если значение true - плагин будет отправлять данные корзины покупателя - $order_bundle
     * @var boolean
     */
    public $send_order;
    /**
     * Корзина покупателя
     * Если массив не пустой и $send_order = true - плагин будет отправлять данные корзины покупателя
     * @var array
     */
    public $order_bundle = [];
    /**
     * Локаль платежной формы
     * @var string
     */
    public $language = 'ru';
    /**
     * ЛОГИРОВАНИЕ
     * Если значение true - плагин будет логировать запросы в ПШ
     * @var boolean
     */
    public $logging;
    /**
     * Путь для лог-файла
     * @var string
     */
    public $log_file_path;
    /**
     * ЛОГИН МЕРЧАНТА
     * @var string
     */
    private $user_name;

    /**
     * ПАРОЛЬ МЕРЧАНТА
     * @var string
     */
    private $password;

    /**
     * ДВУХСТАДИЙНЫЙ ПЛАТЕЖ
     * Если значение true - будет производиться двухстадийный платеж
     * @var boolean
     */
    private $two_stage;

    /**
     * ТЕСТОВЫЙ РЕЖИМ
     * Если значение true - плагин будет работать в тестовом режиме
     *
     * @var boolean
     */
    private $test_mode;

    /**
     * КОНСТРУКТОР КЛАССА
     * Заполнение свойств объекта
     * @param $params
     * @return RBS
     * @internal param string $user_name логин мерчанта
     * @internal param string $password пароль мерчанта
     * @internal param bool $logging логирование
     * @internal param bool $two_stage двухстадийный платеж
     * @internal param bool $test_mode тестовый режим
     */

    public function RBS($params = array())
    {
        foreach ($params as $key => $value) {
            $this->{$key} = $value;
        }
    }

    /**
     * РЕГИСТРАЦИЯ ЗАКАЗА
     * Метод register.do или registerPreAuth.do
     * @param string $order_number номер заказа в магазине
     * @param integer $amount сумма заказа
     * @param string $return_url страница, на которую необходимо вернуть пользователя
     * @return string[]
     */
    function register_order($order_number, $amount, $return_url)
    {
        $data = array(
            'orderNumber' => $order_number,
            'amount' => $amount,
            'returnUrl' => $return_url,
            'language' => $this->language,
            'jsonParams' => json_encode(
                [
                    'CMS:' => $this->cms_name . " " . $this->cms_version,
                    'Module-Version: ' => $this->module_version
                ]
            ),
        );

        if ($this->two_stage) {
            $method = 'registerPreAuth.do';
        } else {
            $method = 'register.do';
        }

        /* Создание и заполнение массива данных заказа для фискализации */
        if ($this->send_order && !empty($this->order_bundle)) {
            /* Заполнение массива данных для запроса c фискализацией */
            $data['orderBundle'] = json_encode($this->order_bundle);
        }

        $response = $this->gateway($method, $data);
        return $response;
    }

    /**
     * ЗАПРОС В ПШ
     * Формирование запроса в платежный шлюз и парсинг JSON-ответа
     * @param string $method метод запроса в ПШ
     * @param string[] $data данные в запросе
     * @return string[]
     */
    private function gateway($method, $data)
    {
//        echo "<pre>";
//        print_r($data);
//        die;

        $data['userName'] = $this->user_name;
        $data['password'] = $this->password;
        $data['language'] = $this->language;

        if ($this->test_mode) {
            $url = self::API_TEST_URL;
        } else {
            $url = self::API_PROD_URL;
        }
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $url . $method,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_POST => true,
            CURLOPT_SSL_VERIFYPEER => false,
            CURLOPT_POSTFIELDS => http_build_query($data, '', '&'),
            CURLOPT_HTTPHEADER => array('CMS: ' . $this->cms_name . ' ' . $this->cms_version, 'Module-Version: ' . $this->module_version),
            CURLOPT_SSLVERSION => 6
        ));
        $response = curl_exec($curl);
        $response = json_decode($response, true);
        if ($this->logging) {
            $this->logger('REQUEST: ' . $url . $method . ': ' . json_encode($data) . ' RESPONSE: ' . json_encode($response));
        }
        curl_close($curl);
        return $response;
    }

    /**
     * ЛОГГЕР
     * Логирование запроса и ответа от ПШ
     * @param string $var
     * @return integer
     */
    private function logger($var)
    {

        $date = "\nDate: " . date('Y-m-d H:i:s') . "\n";
        $result = $var;
        if (is_array($var) || is_object($var)) {
            $result = print_r($var, true);
        }
        $result .= "\n";

        if (empty($this->log_file_path)) {
            $this->log_file_path = dirname($_SERVER['SCRIPT_FILENAME']);
        }
        return error_log($date . $result, 3, $this->log_file_path . '/' . self::log_file);

    }

    /**
     * MDORDER ИЗ FORMURL
     * @param string $url адрес платежной страницы
     * @return string
     */
    public function get_mdOrder_from_url($url)
    {
        $parse = parse_url($url);
        $mdOrder = explode('=', $parse['query']);
        return $mdOrder[1];
    }

    /**
     * СТАТУС ЗАКАЗА ПО ORDER ID
     * Метод getOrderStatusExtended.do
     * @param string $orderId номер заказа в ПШ
     * @return string[]
     */
    public function get_order_status_by_orderId($orderId)
    {
        $data = array('orderId' => $orderId);
        $response = $this->gateway('getOrderStatusExtended.do', $data);
        return $response;
    }

    /**
     * СТАТУС ЗАКАЗА ПО ORDER NUMBER
     * Метод getOrderStatusExtended.do
     * @param string $order_number номер заказа в магазине
     * @return string[]
     */
    public function get_order_status_by_orderNumber($order_number)
    {
        $data = array('orderNumber' => $order_number);
        $response = $this->gateway('getOrderStatusExtended.do', $data);
        return $response;
    }
}


class rbsPayment extends payment
{

    /**
     * @var array $validTaxIdList список поддерживаемых идентификаторов ставок НДС
     * сопоставляем значения стовок от Yandex к rbs (не плодить лишнего в UMI)
     */
    private $validTaxIdList = array(
        2 => 0, 3 => 2, 4 => 3
    );

    /**
     * {@inheritdoc}
     */
    public static function getOrderId()
    {
        return (int)getRequest('shp_orderId');
    }

    /**
     * {@inheritdoc}
     */
    public function validate()
    {
        return true;
    }

    /**
     * {@inheritdoc}
     * Устанавливает заказу статус оплаты "Инициализирована"
     */
    public function process($template = null)
    {
        //$this->order->order();
        $order = $this->order;
        $object = $this->object;

        /**
         * @var emarket $emarket
         */
        $cmsController = cmsController::getInstance();
        $emarket = $cmsController->getModule('emarket');
        /**
         * @var iUmiObject $currency
         */
        $currency = $emarket->getDefaultCurrency();
        $currency = ($currency instanceof iUmiObject) ? $currency->getValue('codename') : 'RUB';

        if ($currency == 'RUR') {
            $currency = 'RUB';
        }

        if (!strlen($object->login) || !strlen($object->password)) {
            throw new publicException(getLabel('error-payment-wrong-settings'));
        }

        $config['user_name'] = $object->login;
        $config['password'] = $object->password;
        $config['two_stage'] = $object->stage;
        $config['test_mode'] = $object->test_mode;
        $config['send_order'] = $object->send_order;
        $config['logging'] = $object->logging;
        $config['cms_name'] = "UMI";
        $config['cms_version'] = '17';
        $config['module_version'] = '2.1';

        // collect orderBundle
        if ($config['send_order']) {
            $config['order_bundle'] = $this->getOrderBundle($order);
        }

        $rbs = new RBS($config);

        $order_number = $this->order->getId();
        $amount = round($this->order->getActualPrice() * 100);

        $return_url = 'http://' . cmsController::getInstance()->getCurrentDomain()->getHost() . '/emarket/gateway/' . $this->order->getId();

        $response = $rbs->register_order("umi_" . $order_number, $amount, $return_url);

        $errorCode = $response['errorCode'];

        if ($errorCode == 0) {
            $params = array();
            $params['formAction'] = $response['formUrl'];
            $params['mdOrder'] = $rbs->get_mdOrder_from_url($response['formUrl']);

            $this->order->setPaymentStatus('initialized');

            list($templateString) = emarket::loadTemplates(
                "emarket/payment/rbs/" . $template,
                "form_block"
            );

            return emarket::parseTemplate($templateString, $params);

        } else {
            throw new privateException("Ошибка #{$response['errorCode']}: {$response['errorMessage']}");
        }


    }

    /**
     * Возвращает данные для печати чека.
     * @param order $order $order заказ
     * @return string json
     * @throws publicException
     */
    protected function getOrderBundle(order $order)
    {

        $customerId = $order->getCustomerId();
        $customerSource = selector::get('object')
            ->id($customerId);

        if (!$customerSource instanceof iUmiObject) {
            throw new publicException(getLabel('error-payment-wrong-customer-id'));
        }
        $customer = new customer($customerSource);

        $date = new DateTime();

        $order_bundle = array(
            'orderCreationDate' => $date->getTimestamp(),
            'customerDetails' => array(
                'email' => $this->getCustomerEmail($customer)
            ),
            'cartItems' => array('items' => $this->getOrderItemInfoList($order))
        );


        if (!is_array($order_bundle)) {
            throw new publicException(getLabel('error-payment-wrong-receipt-info'));
        }

        return $order_bundle;
    }

    /**
     * Возвращает контакт почтовый ящик покупателя
     * @param customer $customer покупатель
     * @return string
     * @throws publicException
     */
    protected function getCustomerEmail(customer $customer)
    {

        $email = $customer->getEmail();

        if (!is_string($email) || empty($email) || !umiMail::checkEmail($email)) {
            throw new publicException(getLabel('error-payment-wrong-customer-email'));
        }
        return $email;
    }

    /**
     * Возвращает информацию о списке товарных наименований
     * @param order $order заказ
     * @return array
     * [
     *      'items' =>  [
     *      ]
     * ]
     * @throws publicException
     */
    protected function getOrderItemInfoList(order $order)
    {
        $orderItemList = $order->getItems();

        if (!is_array($orderItemList) || empty($orderItemList)) {
            throw new publicException(getLabel('error-payment-empty-order'));
        }

        $orderItemDataList = [];

        $itemsCnt = 1;
        foreach ($orderItemList as $orderItem) {
            $orderItemDataList[] = $this->getOrderItemInfo($orderItem, $itemsCnt);
            $itemsCnt++;
        }

        try {
            $deliveryInfo = $this->getDeliveryInfo($order, $itemsCnt);
            $orderItemDataList[] = $deliveryInfo;
        } catch (expectObjectException $e) {
            // noop
        }

        return $orderItemDataList;
    }

    /**
     * Возвращает информацию о товарном наименовании
     * @param orderItem $orderItem товарное наименование
     * @return array
     *
     * [
     *      'quantity' => Количество товара
     *      'price' => [
     *          'amount' => Цена за единицу товара
     *      ],
     *      'tax' => id ставки НДС
     *      'text' => Название товара
     * ]
     */
    protected function getOrderItemInfo(orderItem $orderItem, $itemsCnt)
    {

        $item_price = sprintf('%.2f', $orderItem->getActualPrice()) * 100;
        $item_quantity = sprintf('%.3f', $orderItem->getAmount());

        return [
            'positionId' => $itemsCnt,
            'name' => $orderItem->getName(),
            'itemAmount' => $item_price * $item_quantity,
            'itemPrice' => $item_price,
            'quantity' => array(
                'value' => $item_quantity,
                'measure' => 'piece'
            ),
            'tax' => array(
                'taxType' => $this->getEntityTaxRateId($orderItem->getObject()),
            ),
            'itemCode' => $orderItem->getId()

        ];
    }

    /**
     * Возвращает идентификатор ставки НДС
     * @param iUmiObject $object сущность со ставкой (способ доставки или наименование заказа)
     * @return int
     * @throws publicException
     */
    protected function getEntityTaxRateId(iUmiObject $object)
    {
        $taxRate = selector::get('object')
            ->id($object->getValue('tax_rate_id'));

        if (!$taxRate instanceof iUmiObject) {
            return 0;
            //throw new publicException(getLabel('error-payment-order-item-empty-tax'));
        }

        // в UMI предустановленные значения, что бы не плодить свои и не выносить в настройки, авозьмем их
        $taxRateId = (int) $taxRate->getValue('yandex_id');

        if (!in_array($taxRateId, $this->validTaxIdList)) {
            return 0;
            // throw new publicException(getLabel('error-payment-order-item-empty-tax'));
        }

        return $taxRateId;
    }

    /**
     * Возвращает информацию о доставке заказа
     * @param order $order заказ
     * @return array
     * @throws expectObjectException
     */
    protected function getDeliveryInfo(order $order, $itemsCnt)
    {
        $delivery = selector::get('object')->id(
            $order->getValue('delivery_id')
        );

        if (!$delivery instanceof iUmiObject) {
            throw new expectObjectException(getLabel('error-unexpected-exception'));
        }

        return [
            'positionId' => $itemsCnt,
            'itemPrice' => sprintf('%.2f', $order->getValue('delivery_price')) * 100,
            'itemAmount' => sprintf('%.2f', $order->getValue('delivery_price')) * 100,
            'quantity' => array(
                'value' => sprintf('%.3f', 1),
                'measure' => 'piece'
            ),
            'tax' =>
                array(
                    'taxType' => $this->getEntityTaxRateId($delivery),
                ),
            'name' => $delivery->getName(),
            'itemCode' => $delivery->getId()
        ];
    }

    public function poll()
    {

        $buffer = outputBuffer::current();
        $buffer->clear();
        $buffer->contentType('text/plain');

        $config['user_name'] = $this->object->login;
        $config['password'] = $this->object->password;
        $config['test_mode'] = $this->object->test_mode;
        $config['logging'] = $this->object->logging;

        $rbs = new RBS($config);

        $response = $rbs->get_order_status_by_orderId(getRequest('orderId'));
        $host = 'http://' . cmsController::getInstance()->getCurrentDomain()->getHost();

        if ($response['ErrorCode'] == 0 && (($response['orderStatus'] == 1) || ($response['orderStatus'] == 2))) {
            $this->order->setPaymentStatus('accepted');
            $this->order->commit();
            $host .= '/emarket/purchase/result/successful/';
        } else {
            $host .= '/emarket/purchase/result/fail/';
        }
        $cmsController = cmsController::getInstance();
        $module = $cmsController->getModule("emarket");
        if ($module) {
            $module->redirect($host);
        }
    }


}